<?php

namespace Fastrack\Property;

use Fastrack\Gallery\Gallery;
use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\GridField\GridFieldEditButton;
use SilverStripe\Forms\HiddenField;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBDate;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;
use SilverStripe\Subsites\Model\Subsite;
use SilverStripe\Subsites\State\SubsiteState;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;

/**
 * @property string Title
 * @property string Name
 * @property string Slug
 * @property string Price
 * @property string Url
 * @property DBDate Start
 * @property DBDate End
 * @property string Terms
 * @property bool Active
 * @property bool Featured
 */
class Offer extends DataObject implements PermissionProvider
{
	private static $table_name = 'offer';

	private static $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'Slug' => 'Varchar(255)',
		'Price' => 'Varchar(255)',
		'Url' => 'Varchar(255)',
		'Start' => 'Date',
		'End' => 'Date',
		'Terms' => 'HTMLText',
		'Active' => 'Boolean',
		'Featured' => 'Boolean',
		'SortOrder' => 'Int',
	];

	private static $has_one = [
		'Subsite' => Subsite::class,
	];

	private static $has_many = [
		'Galleries' => Gallery::class,

	];

	private static $many_many = [
		'Properties' => Property::class,
	];

	private static $many_many_extraFields = [
		'Properties' => [
			'OfferSortOrder' => 'Int',
		],
	];

	private static $default_sort = 'SortOrder';

	private static $indexes = [
		'Slug' => true,
	];

	private static $translate = [
		'Title',
		'Name',
		'Terms',
	];

    public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		if (class_exists(Subsite::class)){
			$fields->push(new HiddenField('SubsiteID','SubsiteID', SubsiteState::singleton()->getSubsiteId()));
		}
		$gridConfiguration = GridFieldConfig_RelationEditor::create(100);
		$gridConfiguration->removeComponentsByType(GridFieldAddNewButton::class);
		$gridConfiguration->removeComponentsByType(GridFieldEditButton::class);
		$gridConfiguration->addComponent(new GridFieldAddExistingSearchButton());
		$grid = new GridField('Properties', 'Properties', $this->Properties(), $gridConfiguration);
		$fields->addFieldToTab('Root.Properties', $grid);
		ModelUtilities::hideFields($this, $fields);
		$fields->removeByName('Title');
		$fields->removeByName('SortOrder');
		return $fields;
	}

	public function onBeforeWrite() {
    	$this->Title = $this->Name;
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		Gallery::createDefaultGalleries($this);
		parent::onBeforeWrite();
	}

	public function isValid()
	{
		$today = strtotime(date('Y-m-d'));
		if (!$this->Active) {
			return false;
		}
		if ($this->End && ($today > strtotime($this->End))) {
			return false;
		}
		if ($this->Start && ($today < strtotime($this->Start))) {
			return false;
		}
		return true;
	}

	public function Link()
	{
		$link = new ArrayList([$this->Properties()->first()->OffersLink(), $this->Slug]);
		$this->extend('updateOfferLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function CMSEditLink()
	{
		$admin = Injector::inst()->get(PropertyAdmin::class);
		return Controller::join_links([
			$admin->Link('offer'),
			'EditForm',
			'field',
			'offer',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink()
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function providePermissions()
	{
		$permisions = [
			'MANAGE_OFFERS' => [
				'name' => 'Manage offers',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return $this->isValid();
			}
		}
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

}
